"""
A Common directive, from which all other test directives inherit the shared functions.
"""
# fmt: off
import os

from docutils.parsers.rst import Directive
from sphinx.util import logging
from sphinx_needs.api import make_hashed_id

from sphinxcontrib.test_reports.exceptions import SphinxError, TestReportFileNotSetException
from sphinxcontrib.test_reports.junitparser import JUnitParser

# fmt: on


class TestCommonDirective(Directive):
    """
    Common directive, which provides some shared functions to "real" directives.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.env = self.state.document.settings.env
        self.app = self.env.app
        if not hasattr(self.app, "testreport_data"):
            self.app.testreport_data = {}

        self.test_file = None
        self.results = None
        self.docname = None
        self.test_name = None
        self.test_id = None
        self.test_content = None
        self.test_file_given = None
        self.test_links = None
        self.test_tags = None
        self.test_status = None
        self.collapse = None
        self.need_type = None

        self.log = logging.getLogger(__name__)

    def load_test_file(self):
        """
        Loads the defined test_file under self.test_file.

        ``prepare_basic_options`` must be called first

        :return: None
        """
        if self.test_file is None:
            raise TestReportFileNotSetException("Option test_file must be set.")

        root_path = self.app.config.tr_rootdir
        if not os.path.isabs(self.test_file):
            self.test_file = os.path.join(root_path, self.test_file)
        if not os.path.exists(self.test_file):
            # raise TestReportFileInvalidException('Given test_file path invalid: {}'.format(self.test_file))
            self.log.warning(
                "Given test_file path invalid: {} in {} (Line: {})".format(
                    self.test_file, self.docname, self.lineno
                )
            )
            return None

        if self.test_file not in self.app.testreport_data.keys():
            parser = JUnitParser(self.test_file)
            self.app.testreport_data[self.test_file] = parser.parse()

        self.results = self.app.testreport_data[self.test_file]
        return self.results

    def prepare_basic_options(self):
        """
        Reads and checks the needed basic data like name, id, links, status, ...
        :return: None
        """
        self.docname = self.state.document.settings.env.docname

        self.test_name = self.arguments[0]
        self.test_content = "\n".join(self.content)
        if self.name != "test-report":
            self.need_type = self.app.tr_types[self.name][0]
            self.test_id = self.options.get(
                "id",
                make_hashed_id(
                    self.app, self.need_type, self.test_name, self.test_content
                ),
            )
        else:
            self.test_id = self.options.get("id", None)

        if self.test_id is None:
            raise SphinxError("ID must be set for test-report.")

        self.test_file = self.options.get("file", None)
        self.test_file_given = self.test_file[:]

        self.test_links = self.options.get("links", "")
        self.test_tags = self.options.get("tags", "")
        self.test_status = self.options.get("status", None)

        self.collapse = str(self.options.get("collapse", ""))

        if isinstance(self.collapse, str) and len(self.collapse) > 0:
            if self.collapse.upper() in ["TRUE", 1, "YES"]:
                self.collapse = True
            elif self.collapse.upper() in ["FALSE", 0, "NO"]:
                self.collapse = False
            else:
                raise Exception("collapse attribute must be true or false")
        else:
            self.collapse = getattr(self.app.config, "needs_collapse_details", True)



def tr_link(app, need, needs, test_option, target_option, *args, **kwargs):
    if test_option not in need:
        return ""
    test_opt = need[test_option]

    links = []
    for need_target in needs.values():
        if target_option not in need_target:
            continue

        if (
            test_opt == need_target[target_option] and test_opt is not None and len(test_opt) > 0  # fmt: skip
        ):
            links.append(need_target["id"])

    return links



from pathlib import Path

import pytest


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/custom_types"}],
    indirect=True,
)
def test_custom_types_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir / "index.html").read_text()
    assert "Test-Path" in html
    assert "Test-Container" in html
    assert "Test-Run" in html



"""Pytest conftest module containing common test configuration and fixtures."""
import shutil
from tempfile import mkdtemp

import pytest
from sphinx.testing.path import path

pytest_plugins = "sphinx.testing.fixtures"


def copy_srcdir_to_tmpdir(srcdir, tmp):
    srcdir = path(__file__).parent.abspath() / srcdir
    tmproot = tmp / path(srcdir).basename()
    shutil.copytree(srcdir, tmproot)
    return tmproot


@pytest.fixture(scope="function")
def test_app(make_app, request):
    # We create a temp-folder on our own, as the util-functions from sphinx and pytest make troubles.
    # It seems like they reuse certain-temp names
    sphinx_test_tempdir = path(mkdtemp())

    builder_params = request.param

    # copy plantuml.jar, xml files and json files to current test temdir
    util_files = path(__file__).parent.abspath() / "doc_test/utils"
    shutil.copytree(util_files, sphinx_test_tempdir / "utils")

    # copy test srcdir to test temporary directory sphinx_test_tempdir
    srcdir = builder_params.get("srcdir", None)
    src_dir = copy_srcdir_to_tmpdir(srcdir, sphinx_test_tempdir)

    # return sphinx.testing fixture make_app and new srcdir which in sphinx_test_tempdir
    app = make_app(
        buildername=builder_params.get("buildername", "html"),
        srcdir=src_dir,
        freshenv=builder_params.get("freshenv", None),
        confoverrides=builder_params.get("confoverrides", None),
        status=builder_params.get("status", None),
        warning=builder_params.get("warning", None),
        tags=builder_params.get("tags", None),
        docutilsconf=builder_params.get("docutilsconf", None),
        parallel=builder_params.get("parallel", 0),
    )

    yield app

    # cleanup test temporary directory
    shutil.rmtree(sphinx_test_tempdir, False)



from pathlib import Path

import pytest


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/testsuites_doc"}],
    indirect=True,
)
def test_doc_testsuites_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir, "index.html").read_text()
    assert html

    assert "AnotherTest" in html
    assert "test_Another_getCurrentTime" in html

    assert "TimerCounterImplTest" in html
    assert "CheckTimerFunction_Init" in html

    assert "TimerTest" in html
    assert "test_Timer_getdT" in html


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/many_testsuites_doc"}],
    indirect=True,
)
def test_doc_many_testsuites_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir, "index.html").read_text()
    assert html
    assert "TEST_001_FFDBD67" in html  # suite id
    assert "TEST_001_FFDBD67_81152CFAF6" in html  # case id



import os

import sphinx
from pkg_resources import parse_version
from sphinx.util.console import brown
from sphinx.util.osutil import copyfile, ensuredir

sphinx_version = sphinx.__version__
if parse_version(sphinx_version) >= parse_version("1.6"):
    from sphinx.util import status_iterator  # NOQA Sphinx 1.5

STATICS_DIR_NAME = "_static"


def safe_add_file(filename, app):
    """
    Adds files to builder resources only, if the given filename was not already registered.
    Needed mainly for tests to avoid multiple registration of the same file and therefore also multiple execution
    of e.g. a javascript file during page load.

    :param filename: filename to remove
    :param app: app object
    :return: None
    """
    data_file = filename
    static_data_file = os.path.join("_static", data_file)

    if data_file.split(".")[-1] == "js":
        if (
            hasattr(app.builder, "script_files")
            and static_data_file not in app.builder.script_files  # noqa: W503
        ):
            app.add_js_file(data_file)
    elif data_file.split(".")[-1] == "css":
        if (
            hasattr(app.builder, "css_files")
            and static_data_file not in app.builder.css_files  # noqa: W503
        ):
            app.add_css_file(data_file)
    else:
        raise NotImplementedError(
            "File type {} not support by save_add_file".format(data_file.split(".")[-1])
        )


def safe_remove_file(filename, app):
    """
    Removes a given resource file from builder resources.
    Needed mostly during test, if multiple sphinx-build are started.
    During these tests js/cass-files are not cleaned, so a css_file from run A is still registered in run B.

    :param filename: filename to remove
    :param app: app object
    :return: None
    """
    data_file = filename
    static_data_file = os.path.join("_static", data_file)

    if data_file.split(".")[-1] == "js":
        if (
            hasattr(app.builder, "script_files")
            and static_data_file in app.builder.script_files  # noqa: W503
        ):
            app.builder.script_files.remove(static_data_file)
    elif data_file.split(".")[-1] == "css" and (
        hasattr(app.builder, "css_files")
        and static_data_file in app.builder.css_files  # noqa: W503
    ):

        app.builder.css_files.remove(static_data_file)


# Base implementation from sphinxcontrib-images
# https://github.com/spinus/sphinxcontrib-images/blob/master/sphinxcontrib/images.py#L203
def install_styles_static_files(app, env):
    statics_dir_path = os.path.join(app.builder.outdir, STATICS_DIR_NAME)
    dest_path = os.path.join(statics_dir_path, "sphinx-test-results")

    files_to_copy = ["common.css"]

    # Be sure no "old" css layout is already set
    safe_remove_file("sphinx-test-reports/common.css", app)

    if parse_version(sphinx_version) < parse_version("1.6"):
        global status_iterator
        status_iterator = app.status_iterator

    for source_file_path in status_iterator(
        files_to_copy,
        "Copying static files for sphinx-test-results custom style support...",
        brown,
        len(files_to_copy),
    ):

        if not os.path.isabs(source_file_path):
            source_file_path = os.path.join(
                os.path.dirname(__file__), "css", source_file_path
            )

        if not os.path.exists(source_file_path):
            source_file_path = os.path.join(
                os.path.dirname(__file__), "css", "blank.css"
            )
            print(
                "{} not found. Copying sphinx-internal blank.css".format(
                    source_file_path
                )
            )

        dest_file_path = os.path.join(dest_path, os.path.basename(source_file_path))

        if not os.path.exists(os.path.dirname(dest_file_path)):
            ensuredir(os.path.dirname(dest_file_path))

        copyfile(source_file_path, dest_file_path)

        safe_add_file(os.path.relpath(dest_file_path, statics_dir_path), app)



import os

from lxml import etree, objectify


class JUnitParser:
    def __init__(self, junit_xml, junit_xsd=None):
        self.junit_xml_path = junit_xml

        if junit_xsd is None:
            junit_xsd = os.path.join(os.path.dirname(__file__), "schemas", "JUnit.xsd")
        self.junit_xsd_path = junit_xsd

        self.junit_schema_doc = None
        self.xmlschema = None
        self.valid_xml = None

        if not os.path.exists(self.junit_xml_path):
            raise JUnitFileMissing(
                f"The given file does not exist: {self.junit_xml_path}"
            )
        self.junit_xml_doc = etree.parse(self.junit_xml_path)

        self.junit_xml_string = etree.tostring(self.junit_xml_doc)
        self.junit_xml_object = objectify.fromstring(self.junit_xml_string)
        self.junit_xml_string = str(self.junit_xml_string)

    def validate(self):
        self.junit_schema_doc = etree.parse(self.junit_xsd_path)
        self.xmlschema = etree.XMLSchema(self.junit_schema_doc)
        self.valid_xml = self.xmlschema.validate(self.junit_xml_doc)

        return self.valid_xml

    def parse(self):
        """
        Creates a common python list of object, no matter what information are
        supported by the parsed xml file for test results junit().

        :return: list of test suites as dictionaries
        """

        def parse_testcase(xml_object):

            testcase = xml_object

            tc_dict = {
                "classname": testcase.attrib.get("classname", "unknown"),
                "file": testcase.attrib.get("file", "unknown"),
                "line": int(testcase.attrib.get("line", -1)),
                "name": testcase.attrib.get("name", "unknown"),
                "time": float(testcase.attrib.get("time", -1)),
            }

            # The following data is normally a subnode (e.g. skipped/failure).
            # We integrate it right into the testcase for better handling
            if hasattr(testcase, "skipped"):
                result = testcase.skipped
                tc_dict["result"] = "skipped"
                tc_dict["type"] = result.attrib.get("type", "unknown")
                # tc_dict["text"] = re.sub(r"[\n\t]*", "", result.text)  # Removes newlines  and tabs
                # result.text can be None for pytest xfail test cases
                tc_dict["text"] = result.text or ""
                tc_dict["message"] = result.attrib.get("message", "unknown")
            elif hasattr(testcase, "failure"):
                result = testcase.failure
                tc_dict["result"] = "failure"
                tc_dict["type"] = result.attrib.get("type", "unknown")
                # tc_dict["text"] = re.sub(r"[\n\t]*", "", result.text)  # Removes newlines and tabs
                tc_dict["text"] = result.text
                tc_dict["message"] = ""
            else:
                tc_dict["result"] = "passed"
                tc_dict["type"] = ""
                tc_dict["text"] = ""
                tc_dict["message"] = ""

            if hasattr(testcase, "system-out"):
                tc_dict["system-out"] = testcase["system-out"].text
            else:
                tc_dict["system-out"] = ""

            return tc_dict

        def parse_testsuite(xml_object):

            testsuite = xml_object

            tests = int(testsuite.attrib.get("tests", -1))
            errors = int(testsuite.attrib.get("errors", -1))
            failures = int(testsuite.attrib.get("failures", -1))

            # fmt: off
            skips = int(
                testsuite.attrib.get("skips") or testsuite.attrib.get("skip") or testsuite.attrib.get("skipped") or -1
            )
            # fmt: on

            passed = int(tests - sum(x for x in [errors, failures, skips] if x > 0))

            ts_dict = {
                "name": testsuite.attrib.get("name", "unknown"),
                "tests": tests,
                "errors": errors,
                "failures": failures,
                "skips": skips,
                "passed": passed,
                "time": float(testsuite.attrib.get("time", -1)),
                "testcases": [],
                "testsuite_nested": [],
            }

            # add nested testsuite objects to
            if hasattr(testsuite, "testsuite"):

                for ts in testsuite.testsuite:
                    # dict from inner parse
                    inner_testsuite = parse_testsuite(ts)
                    ts_dict["testsuite_nested"].append(inner_testsuite)

            elif hasattr(testsuite, "testcase"):

                for tc in testsuite.testcase:
                    new_testcase = parse_testcase(tc)
                    ts_dict["testcases"].append(new_testcase)

            return ts_dict

        # main flow starts here

        junit_dict = []

        if self.junit_xml_object.tag == "testsuites":
            for testsuite_xml_object in self.junit_xml_object.testsuite:
                complete_testsuite = parse_testsuite(testsuite_xml_object)
                junit_dict.append(complete_testsuite)
        else:
            complete_testsuite = parse_testsuite(self.junit_xml_object)
            junit_dict.append(complete_testsuite)

        return junit_dict

    def docutils_table(self):
        pass


class JUnitFileMissing(BaseException):
    pass



# from .test_case import TestCase, TestCaseDirective
# from .test_common import TestCommonDirective
# from .test_file import TestFile, TestFileDirective
# from .test_report import TestReport, TestReportDirective
# from .test_suite import TestSuite, TestSuiteDirective



# fmt: off
import os

import sphinx
# from docutils import nodes
from pkg_resources import parse_version
from sphinx_needs.api import add_dynamic_function, add_extra_option, add_need_type

from sphinxcontrib.test_reports.directives.test_case import TestCase, TestCaseDirective
from sphinxcontrib.test_reports.directives.test_env import EnvReport, EnvReportDirective
from sphinxcontrib.test_reports.directives.test_file import TestFile, TestFileDirective
from sphinxcontrib.test_reports.directives.test_report import TestReport, TestReportDirective
from sphinxcontrib.test_reports.directives.test_results import TestResults, TestResultsDirective
from sphinxcontrib.test_reports.directives.test_suite import TestSuite, TestSuiteDirective
from sphinxcontrib.test_reports.environment import install_styles_static_files
from sphinxcontrib.test_reports.functions import tr_link

sphinx_version = sphinx.__version__
if parse_version(sphinx_version) >= parse_version("1.6"):
    from sphinx.util import logging
else:
    import logging

# fmt: on

VERSION = "1.0.2"


def setup(app):
    """
    Setup following directives:
    * test_results
    * test_env
    * test_report
    """
    log = logging.getLogger(__name__)
    log.info("Setting up sphinx-test-reports extension")

    # configurations
    app.add_config_value("tr_rootdir", app.confdir, "html")
    app.add_config_value(
        "tr_file",
        ["test-file", "testfile", "Test-File", "TF_", "#ffffff", "node"],
        "html",
    )
    app.add_config_value(
        "tr_suite",
        ["test-suite", "testsuite", "Test-Suite", "TS_", "#cccccc", "folder"],
        "html",
    )
    app.add_config_value(
        "tr_case",
        ["test-case", "testcase", "Test-Case", "TC_", "#999999", "rectangle"],
        "html",
    )

    # adds option for custom template
    template_dir = os.path.join(
        os.path.dirname(__file__), "directives/test_report_template.txt"
    )
    app.add_config_value("tr_report_template", template_dir, "html")

    app.add_config_value("tr_suite_id_length", 3, "html")
    app.add_config_value("tr_case_id_length", 5, "html")

    # nodes
    app.add_node(TestResults)
    app.add_node(TestFile)
    app.add_node(TestSuite)
    app.add_node(TestCase)
    app.add_node(TestReport)
    app.add_node(EnvReport)

    # directives
    app.add_directive("test-results", TestResultsDirective)
    app.add_directive("test-env", EnvReportDirective)
    app.add_directive("test-report", TestReportDirective)

    # events
    app.connect("env-updated", install_styles_static_files)
    app.connect("config-inited", tr_preparation)
    app.connect("config-inited", sphinx_needs_update)

    return {
        "version": VERSION,  # identifies the version of our extension
        "parallel_read_safe": True,  # support parallel modes
        "parallel_write_safe": True,
    }


def tr_preparation(app, *args):
    """
    Prepares needed vars in the app context.
    """
    if not hasattr(app, "tr_types"):
        app.tr_types = {}

    # Collects the configured test-report node types
    app.tr_types[app.config.tr_file[0]] = app.config.tr_file[1:]
    app.tr_types[app.config.tr_suite[0]] = app.config.tr_suite[1:]
    app.tr_types[app.config.tr_case[0]] = app.config.tr_case[1:]

    app.add_directive(app.config.tr_file[0], TestFileDirective)
    app.add_directive(app.config.tr_suite[0], TestSuiteDirective)
    app.add_directive(app.config.tr_case[0], TestCaseDirective)


def sphinx_needs_update(app, *args):
    """
    sphinx-needs configuration
    """

    # Extra options
    # For details read
    # https://sphinx-needs.readthedocs.io/en/latest/api.html#sphinx_needs.api.configuration.add_extra_option

    add_extra_option(app, "file")
    add_extra_option(app, "suite")
    add_extra_option(app, "case")
    add_extra_option(app, "case_name")
    add_extra_option(app, "case_parameter")
    add_extra_option(app, "classname")
    add_extra_option(app, "time")

    add_extra_option(app, "suites")
    add_extra_option(app, "cases")

    add_extra_option(app, "passed")
    add_extra_option(app, "skipped")
    add_extra_option(app, "failed")
    add_extra_option(app, "errors")
    add_extra_option(app, "result")  # used by test cases only

    # Extra dynamic functions
    # For details about usage read
    # https://sphinx-needs.readthedocs.io/en/latest/api.html#sphinx_needs.api.configuration.add_dynamic_function
    add_dynamic_function(app, tr_link)

    # Extra need types
    # For details about usage read
    # https://sphinx-needs.readthedocs.io/en/latest/api.html#sphinx_needs.api.configuration.add_need_type
    add_need_type(app, *app.config.tr_file[1:])
    add_need_type(app, *app.config.tr_suite[1:])
    add_need_type(app, *app.config.tr_case[1:])



from pathlib import Path

import pytest


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/needs_linking"}],
    indirect=True,
)
def test_doc_testsuites_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir, "index.html").read_text()
    assert html



from docutils import nodes
from docutils.parsers.rst import directives
from sphinx_needs.api import add_need
from sphinx_needs.utils import add_doc

from sphinxcontrib.test_reports.directives.test_common import TestCommonDirective
from sphinxcontrib.test_reports.exceptions import TestReportInvalidOption


class TestCase(nodes.General, nodes.Element):
    pass


class TestCaseDirective(TestCommonDirective):
    """
    Directive for showing test suites.
    """

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        "id": directives.unchanged_required,
        "status": directives.unchanged_required,
        "tags": directives.unchanged_required,
        "links": directives.unchanged_required,
        "collapse": directives.unchanged_required,
        "file": directives.unchanged_required,
        "suite": directives.unchanged_required,
        "case": directives.unchanged_required,
        "classname": directives.unchanged_required,
    }

    final_argument_whitespace = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def run(self, nested=False, suite_count=-1, case_count=-1):
        self.prepare_basic_options()
        self.load_test_file()

        if nested and suite_count >= 0:
            # access n-th nested suite here
            self.results = self.results[0]["testsuites"][suite_count]

        suite_name = self.options.get("suite", None)

        if suite_name is None:
            raise TestReportInvalidOption("Suite not given!")

        case_full_name = self.options.get("case", None)
        class_name = self.options.get("classname", None)
        if case_full_name is None and class_name is None:
            raise TestReportInvalidOption("Case or classname not given!")

        suite = None
        for suite_obj in self.results:

            if nested:  # nested testsuites
                suite = self.results
                break

            elif suite_obj["name"] == suite_name:
                suite = suite_obj
                break

        if suite is None:
            raise TestReportInvalidOption(
                f"Suite {suite_name} not found in test file {self.test_file}"
            )

        case = None

        for case_obj in suite["testcases"]:

            if (
                case_obj["name"] == case_full_name  # noqa: SIM114
                and class_name is None  # noqa: W503
            ):
                case = case_obj
                break

            elif (
                case_obj["classname"] == class_name  # noqa: SIM114
                and case_full_name is None  # noqa: W503
            ):
                case = case_obj
                break

            elif (
                case_obj["name"] == case_full_name
                and case_obj["classname"] == class_name  # noqa: W503
            ):
                case = case_obj
                break

            elif nested and case_count >= 0:
                # access correct case in list
                case = suite["testcases"][case_count]
                break

            elif nested:
                case = case_obj
                break

        if case is None:
            raise TestReportInvalidOption(
                "Case {} with classname {} not found in test file {} "
                "and testsuite {}".format(
                    case_full_name, class_name, self.test_file, suite_name
                )
            )

        result = case["result"]
        content = self.test_content
        if case["text"] is not None and len(case["text"]) > 0:
            content += """

**Text**::

   {}

""".format(
                "\n   ".join([x.lstrip() for x in case["text"].split("\n")])
            )

        if case["message"] is not None and len(case["message"]) > 0:
            content += """

**Message**::

   {}

""".format(
                "\n   ".join([x.lstrip() for x in case["message"].split("\n")])
            )

        if case["system-out"] is not None and len(case["system-out"]) > 0:
            content += """

**System-out**::

   {}

""".format(
                "\n   ".join([x.lstrip() for x in case["system-out"].split("\n")])
            )

        time = case["time"]
        style = "tr_" + case["result"]

        import re

        groups = re.match(r"^(?P<name>[^\[]+)($|\[(?P<param>.*)?\])", case["name"])
        try:
            case_name = groups["name"]
            case_parameter = groups["param"]
        except TypeError:
            case_name = case_full_name
            case_parameter = ""

        if case_parameter is None:
            case_parameter = ""

        main_section = []
        docname = self.state.document.settings.env.docname
        main_section += add_need(
            self.app,
            self.state,
            docname,
            self.lineno,
            need_type=self.need_type,
            title=self.test_name,
            id=self.test_id,
            content=content,
            links=self.test_links,
            tags=self.test_tags,
            status=self.test_status,
            collapse=self.collapse,
            file=self.test_file_given,
            suite=suite["name"],
            case=case_full_name,
            case_name=case_name,
            case_parameter=case_parameter,
            classname=class_name,
            result=result,
            time=time,
            style=style,
        )

        add_doc(self.env, docname)
        return main_section



import hashlib

from docutils import nodes
from docutils.parsers.rst import directives
from sphinx_needs.api import add_need
from sphinx_needs.utils import add_doc

import sphinxcontrib.test_reports.directives.test_suite
from sphinxcontrib.test_reports.directives.test_common import TestCommonDirective
from sphinxcontrib.test_reports.exceptions import TestReportIncompleteConfiguration


class TestFile(nodes.General, nodes.Element):
    pass


class TestFileDirective(TestCommonDirective):
    """
    Directive for showing test results.
    """

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        "id": directives.unchanged_required,
        "status": directives.unchanged_required,
        "tags": directives.unchanged_required,
        "links": directives.unchanged_required,
        "collapse": directives.unchanged_required,
        "file": directives.unchanged_required,
        "auto_suites": directives.flag,
        "auto_cases": directives.flag,
    }

    final_argument_whitespace = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.suite_ids = {}

    def run(self):
        self.prepare_basic_options()
        results = self.load_test_file()

        # Error handling, if file not found
        if results is None:
            main_section = []
            content = nodes.error()
            para = nodes.paragraph()
            text_string = f"Test file not found: {self.test_file}"
            text = nodes.Text(text_string, text_string)
            para += text
            content.append(para)
            main_section.append(content)
            return main_section

        suites = len(self.results)
        cases = sum(int(x["tests"]) for x in self.results)

        passed = sum(x["passed"] for x in self.results)
        skipped = sum(x["skips"] for x in self.results)
        errors = sum(x["errors"] for x in self.results)
        failed = sum(x["failures"] for x in self.results)

        main_section = []
        docname = self.state.document.settings.env.docname
        main_section += add_need(
            self.app,
            self.state,
            docname,
            self.lineno,
            need_type=self.need_type,
            title=self.test_name,
            id=self.test_id,
            content=self.test_content,
            links=self.test_links,
            tags=self.test_tags,
            status=self.test_status,
            collapse=self.collapse,
            file=self.test_file_given,
            suites=suites,
            cases=cases,
            passed=passed,
            skipped=skipped,
            failed=failed,
            errors=errors,
        )

        if (
            "auto_cases" in self.options.keys()
            and "auto_suites" not in self.options.keys()  # noqa W 503
        ):

            raise TestReportIncompleteConfiguration(
                "option auto_cases must be used together with "
                "auto_suites for test-file directives."
            )

        if "auto_suites" in self.options.keys():
            for suite in self.results:
                suite_id = self.test_id
                suite_id += (
                    "_"
                    + hashlib.sha1(suite["name"].encode("UTF-8"))  # noqa: W503
                    .hexdigest()
                    .upper()[: self.app.config.tr_suite_id_length]
                )

                if suite_id not in self.suite_ids:
                    self.suite_ids[suite_id] = suite["name"]
                else:
                    raise Exception(
                        f'Suite ID {suite_id} already exists by {self.suite_ids[suite_id]} ({suite["name"]})'
                    )

                options = self.options
                options["suite"] = suite["name"]
                options["id"] = suite_id

                if "links" not in self.options:
                    options["links"] = self.test_id
                elif self.test_id not in options["links"]:
                    options["links"] = options["links"] + ";" + self.test_id

                arguments = [suite["name"]]
                suite_directive = (
                    sphinxcontrib.test_reports.directives.test_suite.TestSuiteDirective(
                        self.app.config.tr_suite[0],
                        arguments,
                        options,
                        "",
                        self.lineno,  # no content
                        self.content_offset,
                        self.block_text,
                        self.state,
                        self.state_machine,
                    )
                )

                main_section += suite_directive.run()

        add_doc(self.env, docname)

        return main_section



from pathlib import Path

import pytest


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/doc_test_file"}],
    indirect=True,
)
def test_doc_build_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir, "index.html").read_text()
    assert html


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/doc_test_file"}],
    indirect=True,
)
def test_test_file_needs_extra_options_no_warning(test_app):
    import subprocess

    app = test_app

    srcdir = Path(app.srcdir)
    out_dir = srcdir / "_build"

    out = subprocess.run(
        ["sphinx-build", "-M", "html", srcdir, out_dir], capture_output=True
    )

    assert out.returncode == 0

    # Check no warnings
    assert "WARNING" not in out.stdout.decode("utf-8")



# fmt: off
import os

from docutils import nodes
from docutils.parsers.rst import directives

from sphinxcontrib.test_reports.directives.test_common import TestCommonDirective
from sphinxcontrib.test_reports.exceptions import InvalidConfigurationError

# fmt: on


class TestReport(nodes.General, nodes.Element):
    pass


class TestReportDirective(TestCommonDirective):
    """
    Directive for showing test suites.
    """

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        "id": directives.unchanged_required,
        "status": directives.unchanged_required,
        "tags": directives.unchanged_required,
        "links": directives.unchanged_required,
        "collapse": directives.unchanged_required,
        "file": directives.unchanged_required,
    }

    final_argument_whitespace = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def run(self):
        self.prepare_basic_options()
        self.load_test_file()

        # if user provides a custom template, use it
        tr_template = self.app.config.tr_report_template

        template_path = ""

        if os.path.isabs(tr_template):
            template_path = tr_template

        else:
            template_path = os.path.join(self.app.confdir, os.path.relpath(tr_template))

        if not os.path.isfile(template_path):
            raise InvalidConfigurationError(
                "could not find a template file with name {} in conf.py directory".format(
                    template_path
                )
            )

        with open(template_path) as template_file:
            template = "".join(template_file.readlines())

        if self.test_links is not None and len(self.test_links) > 0:
            links_string = f"\n   :links: {self.test_links}"
        else:
            links_string = ""

        template_data = {
            "file": self.test_file,
            "id": self.test_id,
            "file_type": self.app.config.tr_file[0],
            "suite_need": self.app.config.tr_suite[1],
            "case_need": self.app.config.tr_case[1],
            "tags": ";".join([self.test_tags, self.test_id])
            if len(self.test_tags) > 0
            else self.test_id,
            "links_string": links_string,
            "title": self.test_name,
            "content": self.content,
            "template_path": template_path,
        }

        template_ready = template.format(**template_data)
        self.state_machine.insert_input(
            template_ready.split("\n"), self.state_machine.document.attributes["source"]
        )

        return []



from pathlib import Path

import pytest


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/custom_tr_template"}],
    indirect=True,
)
def test_custom_template(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir / "index.html").read_text()

    # assert changed order when creating a testreport html page
    pos1 = html.index("<strong>Imported data</strong>")
    pos2 = html.index('<div class="line">Test suites:')

    assert pos1 < pos2



from pathlib import Path

import pytest


@pytest.mark.parametrize(
    "test_app", [{"buildername": "html", "srcdir": "doc_test/basic_doc"}], indirect=True
)
def test_doc_build_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir, "index.html").read_text()
    assert "<h1>Basic Document" in html
    assert "ASuccessfulTest" in html


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/pytest_6_2"}],
    indirect=True,
)
def test_doc_build_html_for_pytest_6_2(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir, "index.html").read_text(encoding="utf-8")

    assert "Tests: 6" in html
    assert "Failures: 2" in html
    assert "Errors: 0" in html
    assert "Skips: 3" in html



import re
from pathlib import Path
from subprocess import STDOUT, check_output

import pytest
import sphinx

sphinx_version = int(
    sphinx.__version__.split(".")[0] + sphinx.__version__.split(".")[1]
)


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/env_report_doc"}],
    indirect=True,
)
def test_doc_env_report_build_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir / "index.html").read_text()

    assert "<h1>Test-Env report" in html
    assert '<th class="head"><p>Variable</p></th>' in html

    tables = re.findall("(<table .*?</table>)", html, re.DOTALL)

    assert len(tables) == 2

    # All requested data should be in both tables
    assert "name" in tables[0]
    assert "path" in tables[0]
    assert "host" in tables[0]

    assert "name" in tables[1]
    assert "path" in tables[1]
    assert "host" in tables[1]

    # table 0 should contain py35
    assert "py35" in tables[0]
    assert "flake8" not in tables[0]

    # table 1 should contain flake8
    assert "flake8" in tables[1]
    assert "py35" not in tables[1]


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/env_report_doc_raw"}],
    indirect=True,
)
def test_doc_env_report_raw_build_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir / "index.html").read_text()

    tables = re.findall("(<table .*?</table>)", html, re.DOTALL)
    assert len(tables) == 0

    # only requested data should be in the output document
    assert "name" in html
    assert "path" in html
    assert "host" in html
    assert "installed_packages" not in html

    # only requested environment should be present in output document
    assert "py35" in html
    assert "flake8" in html
    assert "pylint" not in html


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/env_report_doc_default"}],
    indirect=True,
)
def test_doc_env_report_default_build_html(test_app):
    app = test_app
    app.build()
    html = Path(app.outdir / "index.html").read_text()

    tables = re.findall("(<table .*?</table>)", html, re.DOTALL)
    assert len(tables) == 3

    # checking if all env from JSON file is present in output or not in table format
    # AS it is not clearm in which order the tablesget printed, we need to check all of them
    # for the needed test case
    assert "pylint" in tables[0] or "pylint" in tables[1] or "pylint" in tables[2]
    assert "py35" in tables[0] or "py35" in tables[1] or "py35" in tables[2]
    assert "flake8" in tables[0] or "flake8" in tables[1] or "flake8" in tables[2]
    assert "py36" not in tables[0] or "py36" not in tables[1] or "py36" not in tables[2]

    # checking if all dataoptions are present in each output table or not
    for table in tables:
        assert "path" in table
        assert "name" in table
        assert "reportversion" in table
        assert "host" in table
        assert "setup" in table
        assert "platform" in table
        assert "test" in table
        assert "toxversion" in table
        assert "python" in table
        assert "installed_packages" in table
        assert "installpkg" in table


@pytest.mark.parametrize(
    "test_app",
    [{"buildername": "html", "srcdir": "doc_test/env_report_warnings"}],
    indirect=True,
)
def test_doc_env_report_warning_build_html(test_app):

    # it should pass all test cases from: test_doc_env_report_build_html
    # generated output data will be stored here
    app = test_app
    output = str(
        check_output(
            ["sphinx-build", "-a", "-E", "-b", "html", app.srcdir, app.outdir],
            stderr=STDOUT,
            universal_newlines=True,
        )
    )
    # app.build()
    html = Path(app.outdir / "index.html").read_text()

    assert "<h1>Test-Env report" in html
    assert '<th class="head"><p>Variable</p></th>' in html

    tables = re.findall("(<table .*?</table>)", html, re.DOTALL)

    assert len(tables) == 2

    # All requested data should be in both tables
    assert "name" in tables[0]
    assert "path" in tables[0]
    assert "host" in tables[0]

    assert "name" in tables[1]
    assert "path" in tables[1]
    assert "host" in tables[1]

    # table 0 should contain py35
    assert "py35" in tables[0]
    assert "flake8" not in tables[0]

    # table 1 should contain flake8
    assert "flake8" in tables[1]
    assert "py35" not in tables[1]

    """
    Taken from solution of this issue:
        https://github.com/useblocks/sphinxcontrib-needs/issues/44
    """

    if sphinx_version > 15:
        assert "WARNING: environment 'defs' is not present in JSON file" in output
        assert "WARNING: option 'abc' is not present in JSON file" in output
    else:
        assert "environment 'defs' is not present in JSON file" in output
        assert "option 'abc' is not present in JSON file" in output



from sphinx.errors import SphinxError, SphinxWarning


class TestReportFileNotSetException(SphinxError):
    """
    Raised if a needed test_file path is not given in directive.
    """


class TestReportFileInvalidException(SphinxError):
    """
    Raised if the given path is not valid.
    """


class TestReportInvalidOption(SphinxError):
    """
    Raised if an option is not given or invalid.
    """


class TestReportIncompleteConfiguration(SphinxWarning):
    """
    Raised if given arguments / options are not correct configured
    """


class InvalidConfigurationError(SphinxError):
    """
    Raised if wrong values given in conf.py
    """



import copy
import json
import os

import sphinx
from docutils import nodes
from docutils.parsers.rst import Directive, directives
from pkg_resources import parse_version

sphinx_version = sphinx.__version__
if parse_version(sphinx_version) >= parse_version("1.6"):
    from sphinx.util import logging
else:
    import logging

    logging.basicConfig()
logger = logging.getLogger(__name__)


class EnvReport(nodes.General, nodes.Element):
    pass


class EnvReportDirective(Directive):
    """
    Directive for showing test results.
    """

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        "env": directives.unchanged_required,
        "data": directives.unchanged_required,
        "raw": directives.flag,
    }

    final_argument_whitespace = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.data_option = self.options.get("data", None)
        self.environments = self.options.get("env", None)

        if self.environments is not None:
            self.req_env_list_cpy = self.environments.split(",")
            self.req_env_list = []
            for element in self.req_env_list_cpy:
                if len(element) != 0:
                    self.req_env_list.append(element.lstrip().rstrip())
        else:
            self.req_env_list = None

        if self.data_option is not None:
            self.data_option_list_cpy = self.data_option.split(",")
            self.data_option_list = []
            for element in self.data_option_list_cpy:
                if len(element) != 0:
                    self.data_option_list.append(element.rstrip().lstrip())
        else:
            self.data_option_list = None

        self.header = ("Variable", "Data")
        self.colwidths = (1, 1)

    def run(self):
        env = self.state.document.settings.env

        json_path = self.arguments[0]
        root_path = env.app.config.tr_rootdir
        if not os.path.isabs(json_path):
            json_path = os.path.join(root_path, json_path)

        if not os.path.exists(json_path):
            raise JsonFileNotFound(f"The given file does not exist: {json_path}")

        with open(json_path) as fp_json:
            try:
                results = json.load(fp_json)
            except ValueError:
                raise InvalidJsonFile(
                    "The given file {} is not a valid JSON".format(
                        json_path.split("/")[-1]
                    )
                )

        # check to see if environment is present in JSON or not
        if self.req_env_list is not None:
            not_present_env = []
            for req_env in self.req_env_list:
                if req_env not in results:
                    not_present_env.append(req_env)
            for not_env in not_present_env:
                self.req_env_list.remove(not_env)
                logger.warning(f"environment '{not_env}' is not present in JSON file")
            del not_present_env

        # Construction idea taken from http://agateau.com/2015/docutils-snippets/
        main_section = []

        if self.req_env_list is None and "raw" not in self.options:
            for enviro in results:
                main_section += self._crete_table_b(enviro=enviro, results=results)

        elif "raw" not in self.options and self.req_env_list is not None:
            for req_env in self.req_env_list:
                main_section += self._crete_table_b(enviro=req_env, results=results)

        elif "raw" in self.options and self.req_env_list is None:
            for enviro in results:
                # data option handling
                temp_dict = copy.deepcopy(results)
                temp_dict2 = copy.deepcopy(results)
                if self.data_option_list is not None:
                    for opt in temp_dict[enviro]:
                        if opt not in self.data_option_list:
                            del temp_dict2[enviro][opt]
                    # option check
                    for opt in self.data_option_list:
                        if opt not in temp_dict2[enviro]:
                            logger.warning(
                                f"option '{opt}' is not present in JSON file"
                            )

                del temp_dict

                section = nodes.section()
                section += nodes.title(text=enviro)
                results_string = json.dumps(temp_dict2[enviro], indent=4)
                code_block = nodes.literal_block(results_string, results_string)
                code_block["language"] = "json"
                section += code_block  # nodes.literal_block(results, results)
                main_section += section  # nodes.literal_block(enviro, results[enviro])
                del temp_dict2

        elif "raw" in self.options and self.req_env_list is not None:
            for enviro in self.req_env_list:
                # data option handling
                temp_dict = copy.deepcopy(results)
                temp_dict2 = copy.deepcopy(results)
                if self.data_option_list is not None:
                    for opt in temp_dict[enviro]:
                        if opt not in self.data_option_list:
                            del temp_dict2[enviro][opt]

                # option check
                for opt in self.data_option_list:
                    if opt not in temp_dict2[enviro]:
                        logger.warning(
                            "option '{}' is not present in '{}' environment file".format(
                                opt, enviro
                            )
                        )

                del temp_dict

                section = nodes.section()
                section += nodes.title(text=enviro)
                results_string = json.dumps(temp_dict2[enviro], indent=4)
                code_block = nodes.literal_block(results_string, results_string)
                code_block["language"] = "json"
                section += code_block
                main_section += section
                del temp_dict2

        return main_section

    def _crete_table_b(self, enviro, results):
        main_section = []
        section = nodes.section()
        section += nodes.title(text=enviro)

        table = nodes.table()
        section += table

        tgroup = nodes.tgroup(cols=len(self.header))
        table += tgroup
        for colwidth in self.colwidths:
            tgroup += nodes.colspec(colwidth=colwidth)

        thead = nodes.thead()
        tgroup += thead
        thead += self._create_rows(self.header)

        tbody = nodes.tbody()
        tgroup += tbody
        all_data = results[enviro]

        data_option = copy.deepcopy(self.data_option_list)
        for data in all_data:
            if data_option is not None:
                if data in data_option:
                    data_option.remove(data)
                    tbody += self._create_rows((data, all_data[data]))
            else:
                tbody += self._create_rows((data, all_data[data]))

        main_section += section

        # data option check
        if data_option is not None:
            if len(data_option) != 0:
                for opt in data_option:
                    logger.warning(f"option '{opt}' is not present in JSON file")
            del data_option

        return main_section

    def _create_rows(self, row_cells):
        row = nodes.row()
        for cell in row_cells:
            entry = nodes.entry()
            row += entry
            if isinstance(cell, (list, dict)):
                results_string = json.dumps(cell, indent=4)
                code_block = nodes.literal_block(results_string, results_string)
                code_block["language"] = "json"
                entry += code_block
            else:
                entry += nodes.paragraph(text=cell)
        return row


class InvalidJsonFile(BaseException):
    pass


class JsonFileNotFound(BaseException):
    pass


class InvalidEnvRequested(BaseException):
    pass



import hashlib

from docutils import nodes
from docutils.parsers.rst import directives
from sphinx_needs.api import add_need
from sphinx_needs.utils import add_doc

import sphinxcontrib.test_reports.directives.test_case
from sphinxcontrib.test_reports.directives.test_common import TestCommonDirective
from sphinxcontrib.test_reports.exceptions import TestReportInvalidOption


class TestSuite(nodes.General, nodes.Element):
    pass


class TestSuiteDirective(TestCommonDirective):
    """
    Directive for showing test suites.
    """

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        "id": directives.unchanged_required,
        "status": directives.unchanged_required,
        "tags": directives.unchanged_required,
        "links": directives.unchanged_required,
        "collapse": directives.unchanged_required,
        "file": directives.unchanged_required,
        "suite": directives.unchanged_required,
    }

    final_argument_whitespace = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.case_ids = []

    def run(self, nested=False, count=-1):
        self.prepare_basic_options()
        self.load_test_file()

        if nested:
            # access n-th nested suite here
            self.results = self.results[0]["testsuite_nested"]

        suite_name = self.options.get("suite", None)

        if suite_name is None:
            raise TestReportInvalidOption("Suite not given!")

        suite = None
        for suite_obj in self.results:
            if suite_obj["name"] == suite_name:
                suite = suite_obj
                break

            elif nested:  # access correct nested testsuite here
                suite = self.results[count]
                break

        if suite is None:
            raise TestReportInvalidOption(
                f"Suite {suite_name} not found in test file {self.test_file}"
            )

        cases = suite["tests"]

        passed = suite["passed"]
        skipped = suite["skips"]
        errors = suite["errors"]
        failed = suite["failures"]

        main_section = []
        docname = self.state.document.settings.env.docname
        main_section += add_need(
            self.app,
            self.state,
            docname,
            self.lineno,
            need_type=self.need_type,
            title=self.test_name,
            id=self.test_id,
            content=self.test_content,
            links=self.test_links,
            tags=self.test_tags,
            status=self.test_status,
            collapse=self.collapse,
            file=self.test_file_given,
            suite=suite["name"],
            cases=cases,
            passed=passed,
            skipped=skipped,
            failed=failed,
            errors=errors,
        )

        # TODO double nested logic
        # nested testsuite present, if testcases are present -> reached most inner testsuite
        access_count = 0
        if len(suite_obj["testcases"]) == 0:

            for suite in suite_obj["testsuite_nested"]:

                suite_id = self.test_id
                suite_id += (
                    "_"
                    + hashlib.sha1(suite["name"].encode("UTF-8"))  # noqa: W503
                    .hexdigest()
                    .upper()[: self.app.config.tr_suite_id_length]
                )

                options = self.options
                options["suite"] = suite["name"]
                options["id"] = suite_id

                if "links" not in self.options:
                    options["links"] = self.test_id
                elif self.test_id not in options["links"]:
                    options["links"] = options["links"] + ";" + self.test_id

                arguments = [suite["name"]]
                suite_directive = (
                    sphinxcontrib.test_reports.directives.test_suite.TestSuiteDirective(
                        self.app.config.tr_suite[0],
                        arguments,
                        options,
                        "",
                        self.lineno,  # no content
                        self.content_offset,
                        self.block_text,
                        self.state,
                        self.state_machine,
                    )
                )

                is_nested = len(suite_obj["testsuites"]) > 0

                # create suite_directive for each nested suite, directive appends content in html files
                # access_count keeps track of which nested testsuite to access in the directive
                main_section += suite_directive.run(nested=True, count=access_count)
                access_count += 1

        # suite has testcases
        if "auto_cases" in self.options.keys() and len(suite_obj["testcases"]) > 0:

            case_count = 0

            for case in suite["testcases"]:
                case_id = self.test_id
                case_id += (
                    "_"
                    + hashlib.sha1(  # noqa: W503
                        case["classname"].encode("UTF-8") + case["name"].encode("UTF-8")
                    )
                    .hexdigest()
                    .upper()[: self.app.config.tr_case_id_length]
                )

                if case_id not in self.case_ids:
                    self.case_ids.append(case_id)
                else:
                    raise Exception(f"Case ID exists: {case_id}")

                # We need to copy self.options, otherwise it gets updated and sets same values
                # for all testsuites.
                options = self.options.copy()

                options["case"] = case["name"]
                options["classname"] = case["classname"]
                options["id"] = case_id

                if "links" not in self.options:
                    options["links"] = self.test_id
                elif self.test_id not in options["links"]:
                    options["links"] = options["links"] + ";" + self.test_id

                arguments = [case["name"]]
                case_directive = (
                    sphinxcontrib.test_reports.directives.test_case.TestCaseDirective(
                        self.app.config.tr_case[0],
                        arguments,
                        options,
                        "",
                        self.lineno,  # no content
                        self.content_offset,
                        self.block_text,
                        self.state,
                        self.state_machine,
                    )
                )

                is_nested = len(suite_obj["testsuite_nested"]) > 0 or nested

                # depending if nested or not, runs case directive to add content to testcases
                # count is for correct suite access, if multiple present, case_count is for correct case access
                main_section += case_directive.run(is_nested, count, case_count)

                if is_nested:
                    case_count += 1

        add_doc(self.env, docname)

        return main_section



#!/usr/bin/env python

import os

from setuptools import find_packages, setup

requires = ["sphinx>=4.0", "lxml", "sphinx-needs>=1.0.1"]

with open(os.path.join(os.path.dirname(__file__), "README.rst")) as file:

    setup(
        name="sphinx-test-reports",
        # Update also test_reports.py, conf.py and changelog!
        version="1.0.2",
        url="http://github.com/useblocks/sphinx-test-reports",
        download_url="http://pypi.python.org/pypi/sphinx-test-reports",
        license="MIT",
        author="team useblocks",
        author_email="info@useblocks.com",
        description="Sphinx extension for showing test results and test environment "
        "information inside sphinx documentations",
        long_description=file.read(),
        zip_safe=False,
        classifiers=[
            "Framework :: Sphinx",
            "Framework :: Sphinx :: Extension",
            "Development Status :: 4 - Beta",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Topic :: Documentation",
        ],
        platforms="any",
        packages=find_packages(),
        include_package_data=True,
        install_requires=requires,
        namespace_packages=["sphinxcontrib"],
    )



import os

from docutils import nodes
from docutils.parsers.rst import Directive

from sphinxcontrib.test_reports.junitparser import JUnitParser


class TestResults(nodes.General, nodes.Element):
    pass


class TestResultsDirective(Directive):
    """
    Directive for showing test results.
    """

    has_content = True
    required_arguments = 1
    optional_arguments = 0

    final_argument_whitespace = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.header = ("class", "name", "status", "reason")
        self.colwidths = (1, 1, 1, 2)

    def run(self):
        env = self.state.document.settings.env

        xml_path = self.arguments[0]
        root_path = env.app.config.tr_rootdir
        if not os.path.isabs(xml_path):
            xml_path = os.path.join(root_path, xml_path)
        parser = JUnitParser(xml_path)
        results = parser.parse()

        # Construction idea taken from http://agateau.com/2015/docutils-snippets/

        main_section = []

        for testsuite in results:
            section = nodes.section()
            section += nodes.title(text=testsuite["name"])
            section += nodes.paragraph(
                text="Tests: {tests}, Failures: {failure}, Errors: {error}, "
                "Skips: {skips}".format(
                    tests=testsuite["tests"],
                    failure=testsuite["failures"],
                    error=testsuite["errors"],
                    skips=testsuite["skips"],
                )
            )
            section += nodes.paragraph(
                text="Time: {time}".format(time=testsuite["time"])
            )

            table = nodes.table()
            section += table

            tgroup = nodes.tgroup(cols=len(self.header))
            table += tgroup
            for colwidth in self.colwidths:
                tgroup += nodes.colspec(colwidth=colwidth)

            thead = nodes.thead()
            tgroup += thead
            thead += self._create_table_row(self.header)

            tbody = nodes.tbody()
            tgroup += tbody
            for testcase in testsuite["testcases"]:
                tbody += self._create_testcase_row(testcase)

            main_section += section

        return main_section

    def _create_testcase_row(self, testcase):
        row_cells = (
            testcase["classname"],
            testcase["name"],
            testcase["result"],
            "\n\n".join(
                [
                    testcase["message"] if testcase["message"] != "unknown" else "",
                    testcase["text"] if testcase["text"] is not None else "",
                ]
            ),
        )

        row = nodes.row(classes=["tr_" + testcase["result"]])
        for index, cell in enumerate(row_cells):
            entry = nodes.entry(
                classes=["tr_" + testcase["result"], self.header[index]]
            )
            row += entry
            entry += nodes.paragraph(text=cell, classes=["tr_" + testcase["result"]])
        return row

    def _create_table_row(self, row_cells):
        row = nodes.row()
        for cell in row_cells:
            entry = nodes.entry()
            row += entry
            entry += nodes.paragraph(text=cell)
        return row



import os

xml_path = os.path.join(os.path.dirname(__file__), "doc_test/utils", "xml_data.xml")
xml_pytest_path = os.path.join(
    os.path.dirname(__file__), "doc_test/utils", "pytest_data.xml"
)
xml_pytest51_path = os.path.join(
    os.path.dirname(__file__), "doc_test/utils", "pytest_data_5_1.xml"
)
xml_pytest62_path = os.path.join(
    os.path.dirname(__file__), "doc_test/utils", "pytest_data_6_2.xml"
)

xml_nose_path = os.path.join(
    os.path.dirname(__file__), "doc_test/utils", "nose_data.xml"
)


def test_init_parser():
    from sphinxcontrib.test_reports.junitparser import JUnitParser

    parser = JUnitParser(xml_path)

    assert parser is not None


def test_xml_object():
    from sphinxcontrib.test_reports.junitparser import JUnitParser

    parser = JUnitParser(xml_path)
    obj = parser.junit_xml_object
    assert obj.tag == "testsuite"
    assert len(obj.testcase) == 3
    assert obj.testcase[2].failure.text == " details about failure "


def test_parse_easy_xml():
    from sphinxcontrib.test_reports.junitparser import JUnitParser

    parser = JUnitParser(xml_path)
    assert hasattr(parser, "parse")
    results = parser.parse()

    assert len(results) == 1
    assert results[0]["name"] == "unknown"
    assert results[0]["tests"] == 3
    assert results[0]["testcases"][0]["name"] == "ASuccessfulTest"
    assert results[0]["testcases"][0]["classname"] == "foo1"


def test_parse_nosetest_xml():
    from sphinxcontrib.test_reports.junitparser import JUnitParser

    parser = JUnitParser(xml_nose_path)
    assert hasattr(parser, "parse")
    results = parser.parse()

    assert len(results) == 1
    assert results[0]["name"] == "nosetests"
    assert results[0]["tests"] == 5
    assert results[0]["errors"] == 0
    assert results[0]["failures"] == 0
    assert results[0]["skips"] == 0
    assert results[0]["time"] == -1
    assert results[0]["testcases"][0]["name"] == "test_doc_build_html"
    assert results[0]["testcases"][0]["classname"] == "test_basic_doc"
    assert results[0]["testcases"][0]["time"] == 0.283


def test_parse_pytest_xml():
    from sphinxcontrib.test_reports.junitparser import JUnitParser

    parser = JUnitParser(xml_pytest_path)
    assert hasattr(parser, "parse")
    results = parser.parse()

    assert len(results) == 1
    assert results[0]["name"] == "pytest"
    assert results[0]["tests"] == 10
    assert results[0]["errors"] == 0
    assert results[0]["failures"] == 0
    assert results[0]["skips"] == 10
    assert results[0]["time"] == 0.054
    assert results[0]["testcases"][0]["name"] == "FLAKE8"
    assert results[0]["testcases"][0]["classname"] == "setup"
    assert results[0]["testcases"][0]["time"] == 0.000252246856689
    assert results[0]["testcases"][0]["line"] == -1
    assert results[0]["testcases"][0]["file"] == "setup.py"


def test_parse_pytest_51_xml():
    from sphinxcontrib.test_reports.junitparser import JUnitParser

    parser = JUnitParser(xml_pytest51_path)
    assert hasattr(parser, "parse")
    results = parser.parse()

    assert len(results) == 1
    assert results[0]["name"] == "pytest"


def test_parse_pytest_61_gets_test_suite_attributes():
    from sphinxcontrib.test_reports.junitparser import JUnitParser

    parser = JUnitParser(xml_pytest62_path)
    test_suites = parser.parse()

    assert len(test_suites) == 1
    test_suite = test_suites[0]

    assert test_suite["name"] == "pytest62"
    assert test_suite["tests"] == 6
    assert test_suite["errors"] == 0
    assert test_suite["failures"] == 2
    assert test_suite["skips"] == 3
    assert test_suite["passed"] == 1
    assert test_suite["time"] == 4.088
